/*
 *$Id: param_types.cpp,v 1.16 2001/12/21 15:09:43 amirs Exp $
 */
#include <iostream>
#include <assert.h>

#include "param_types.h"
#include "tokens_to_types.h"
#include "logger.h"

ostream& operator<<(ostream& os, const value_type_t& vt)
{
    const string& dtstr = tokens::get_data_type_str(vt.type);
    os<<"'"<<dtstr<<": ";
    
    switch(vt.type){
    case(DT_BOOL):os<<vt.val.b;
	break;
    case(DT_CHAR):os<<vt.val.c;
	break;
    case(DT_INT):os<<vt.val.i;
	break;
    case(DT_UINT):os<<vt.val.ui;
	break;
    case(DT_SHORT):os<<vt.val.s;
	break;
    case(DT_USHORT):os<<vt.val.us;
	break;
    case(DT_LONG):os<<vt.val.l;
	break;
    case(DT_ULONG):os<<vt.val.ul;
	break;
    case (DT_FLOAT):os<<vt.val.f;
	break;
    case (DT_STRING):os<<(vt.val.str) ? vt.val.str : "NULL";
	break;
    case (DT_LAST_TYPE): /* fallthrough */
    default:
	break;
	/* TODO: what to print here? the data type is undefined
	   so the value has no meaning */ 
    }

    os<<"'";
    return os;
}

ostream& operator<<(ostream& os, const process_field_t& proc)
{
    return os<<tokens::get_process_field_str(proc);
}


filter_node_t* 
value_type_t_to_filter_node(const value_type_t& val)
{
    switch (val.type){
    case DT_BOOL:   return make_bool_val(val.val.b); break;
    case DT_CHAR:   return make_char_val(val.val.c); break;
    case DT_INT:    return make_int_val(val.val.i); break;
    case DT_UINT:   return make_uint_val(val.val.ui); break;
    case DT_SHORT:  return make_short_val(val.val.s); break;
    case DT_USHORT: return make_ushort_val(val.val.us); break;
    case DT_LONG:   return make_long_val(val.val.l); break;
    case DT_ULONG:  return make_ulong_val(val.val.ul); break;
    case DT_FLOAT:  return make_float_val(val.val.f); break;
    case DT_STRING: return make_string_val(val.val.str); break;
    case DT_LAST_TYPE: 
    default:
	logger::log(logger::LOG_LEVEL_ERROR,
		    "bad data type in value_type_t_to_filter_node - %d\n",
		    val.type);
	return NULL;
    }
}

filter_node_t* 
operator_to_filter_node(oper_type_t op, filter_node_t* t1, filter_node_t* t2)
{
    switch(op){
    case OT_EQUAL:           return make_equal(t1, t2);
    case OT_LESS:            return make_less_then(t1, t2);
    case OT_PATTERN_MATCH:   return make_pattern_match(t1, t2);
    case OT_GREATER:         return make_greater_then(t1, t2); 
    case OT_LESS_EQUAL:      return make_less_or_equal(t1, t2);
    case OT_GREATER_EQUAL:   return make_greater_or_equal(t1, t2); 
    case OT_NOT_EQUAL:       return make_not_equal(t1, t2); 
    case OT_PLUS:            return make_math_plus(t1, t2);
    case OT_MINUS:           return make_math_minus(t1, t2);
    case OT_BIT_SHIFT_LEFT:  return make_bit_left_shift(t1, t2);
    case OT_BIT_SHIFT_RIGHT: return make_bit_right_shift(t1, t2);
    case OT_BIT_AND:         return make_bit_and(t1, t2);  
    case OT_BIT_OR:          return make_bit_or(t1, t2);
    case OT_BIT_XOR:         return make_bit_xor(t1, t2);
    case OT_BIT_NOT:         return make_bit_not(t1);
    case OT_LOGIC_AND:       return make_logical_and(t1, t2);
    case OT_LOGIC_OR:        return make_logical_or(t1, t2);
    case OT_LOGIC_NOT:       return make_logical_not(t1);
    case OT_LAST_OPERATOR:     
    default:
	logger::log(logger::LOG_LEVEL_ERROR,
		    "unimplemented or unknown op '%d' in operator_to_filter_node\n",
		    (int)op);
	return NULL;
    }
}

filter_node_t* 
process_field_t_to_var(process_field_t proc)
{
    /* TODO: more matches! */
    switch(proc){
    case PT_UID:  return make_var_val(VT_PROC_UID, DT_USHORT); /* TODO: get the actual dt size */
    case PT_GID:  return make_var_val(VT_PROC_GID, DT_USHORT); 
    case PT_COMM: return make_var_val(VT_PROC_COMM, DT_STRING);
    case PT_PCOMM: return make_var_val(VT_PROC_PCOMM, DT_STRING);
    case PT_PID:  return make_var_val(VT_PROC_PID, DT_LONG);
    case PT_PPID: return make_var_val(VT_PROC_PPID, DT_LONG);
    case PT_EUID: return make_var_val(VT_PROC_EUID, DT_USHORT);
    case PT_SUID: return make_var_val(VT_PROC_SUID, DT_USHORT);
    case PT_EGID: return make_var_val(VT_PROC_EGID, DT_USHORT);
    case PT_SGID: return make_var_val(VT_PROC_SGID, DT_USHORT);
    case PT_LAST_PROCESS_FIELD: /* fallthrough */
    default:
	logger::log(logger::LOG_LEVEL_ERROR,
		    "bad or unimplemented process conditional "
	            "in process_field_t_to_var: %d\n",
		    (int)proc);
	return NULL;
    }
}
