/*
 *$Id: rule.h,v 1.12 2001/12/09 19:53:48 chooga Exp $
 */
/* This class represents each rule the daemon builds from the config file.
 * @author: mulix@actcom.co.il
 */

#ifndef _RULE_H_
#define _RULE_H_

#include <list>
#include <string>

//for conditional clauses in the the rule
#include "param_conditional.h"
#include "process_conditional.h"

using namespace std;

//for identifying this rule
typedef unsigned int rule_identifier;


//we identify a syscall by its index in the syscall table,
//for the time being.
typedef unsigned int syscall_identifier;

class rule{
 public:
    //constructor- we give the the rule a unique identifier
    rule(const parser& p);
    //destructor
    ~rule();

    // copy constructor
    rule(const rule& r);

    //is_valid return true if a rule is valid an false otherwise. 
    bool validate() const;

    // set/get methods for line numbers of rule and its data.
    void set_rule_line_num(int line_num) {_rule_line_num = line_num;}
    int get_rule_line_num() {return _rule_line_num;}
    void set_rule_data_line_num(int line_num) {_rule_data_line_num = line_num;}
    int get_rule_data_line_num() {return _rule_data_line_num;}

    //get method for the rule_id
    rule_identifier get_rule_id() const {return _rule_id;}

    // set get methods for rule's "when" (before/after syscall invocation)
    void set_rule_when(rule_when_t when){_rule_when = when;}
    rule_when_t get_rule_when() const {return _rule_when;} 

    //set get methods for syscall
    void set_syscall_identifier(const string& str);
    syscall_identifier get_syscall_identifier() const 
	{ return _syscall_id;};

    void set_syscall_name(const string& str){_syscall_name = str;}
    string get_syscall_name() const {return _syscall_name;} 

    //get_param_conditional allocates a new param_conditional object
    //adds it to the rule's list of conditionals and returns it. 
    param_conditional* new_param_conditional();

    //get_process_conditional allocates a new process_conditional object
    //adds it to the rule's list of conditionals and returns it. 
    process_conditional* new_process_conditional();

    //set get methods for the action
    void set_action(const string& str);
    parser_action_t get_action(void) const {return _act;}

    //get set methods for the rule name. 
    void set_rule_name(const string& str){_rule_name = str;}
    string get_rule_name() const {return _rule_name;} 

    void set_condition_matching(const string& str);
    condition_matching_t get_condition_matching() const {return _cond_match;}

    // get the syscall number
    unsigned int get_syscall_num() const;

    void set_filter_expression(filter_node_t* exp);

    // transform this rule into a filter_node 
    filter_node_t* filter_node() const;
     
    // get the rule's action
    action_t* get_action_t(const string& log_format) const;

    //have the rule write itself to an ostream.
    friend ostream& operator<<(ostream& os, const rule& r);

    //once we have all of the rule info from the user, we need 
    //to deduce some ourselves. 
    bool finalize(void);

 private:
    /* members */
    rule_identifier _rule_id;
    int _rule_line_num;
    int _rule_data_line_num;
    string _rule_name;
    rule_when_t _rule_when;
    syscall_identifier _syscall_id;
    string _syscall_name;
    list<conditional_intf*> _conds;
    parser_action_t _act;
    condition_matching_t _cond_match;
    filter_node_t* _filter_expr;

    /* static members */
    static rule_identifier last_rule_id;

    const parser& _p;

    bool conds_valid() const;
};

#endif //_RULE_H_
