/*
 * $Id: sct_ctrl_lib.h,v 1.9 2001/12/09 19:55:06 chooga Exp $
 */

#ifndef SCT_CTRL_LIB_H_
#define SCT_CTRL_LIB_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "sct_rules/rule_engine_public.h"

/* get the module's interface */
#include "sct_rules_module/module_interface.h"

/* the opaque pointer all sct_ctrl_lib operations use */
typedef struct sct_ctrl_type* sct_ctrl_t;

/* initialize the library.                 */
/* return an sct_ctrl_t, or NULL if failed */
sct_ctrl_t
sct_ctrl_init(void);

/* shutdown the library.                   */
/* return 0 on failure, non 0 on success   */
int 
sct_ctrl_cleanup(sct_ctrl_t ctrl);

/* add a rule with the given rule id to the given syscall,    */
/* and the given 'when' ('before' or 'after' syscall invoke)  */
/* where the rule is made of this filter and this action      */
/* and 'enable rule' specifies if the rule should be          */
/* immediately enabled or not                                 */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_add_rule(sct_ctrl_t ctrl, int rule_id, rule_when_t when,
		  int syscall_id, 
		  filter_node_t* filter, action_t* act, 
		  int enable_rule);

/* delete from the given syscall the rule with this rule id   */
/* and the given 'when' ('before' or 'after' syscall invoke)  */
/* return 0 on failure, non 0 on success                      */
int
sct_ctrl_del_rule(sct_ctrl_t ctrl,
		  int rule_id,
		  rule_when_t when,
		  int syscall_id);


/* enable the rule with this id for this syscall              */
/* NOTE: currently NOT IMPLEMENTED                            */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_enable_rule(sct_ctrl_t ctrl,
		     int rule_id,
		     rule_when_t when,
		     int syscall_id);

/* disable the rule with this id for this syscall             */
/* NOTE: currently NOT IMPLEMENTED                            */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_disable_rule(sct_ctrl_t ctrl,
		      int rule_id,
		      rule_when_t when,
		      int syscall_id);

/* delete all of the rules for this system call.              */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_del_all_syscall_rules(sct_ctrl_t ctrl, int syscall_id);

/* delete all rules for ALL syscalls.                         */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_del_all_rules(sct_ctrl_t ctrl);

/* print all of the current rules to the system's log file    */
/* return 0 on failure, non 0 on success                      */
int
sct_ctrl_print_all_rules(sct_ctrl_t ctrl);

#ifdef __cplusplus
}
#endif

#endif /* SCT_CTRL_LIB_H_ */
