/*
 *$Id: main.c,v 1.16 2001/12/10 23:42:20 chooga Exp $
 */
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>


#include "rule_engine_public.h"
#include "filter_vars_public.h"

/* print the filter, evaluate it, print the results. */
exp_error_t
eval_and_print_filter_and_result(filter_node_t* f, filter_node_t* result)
{
    exp_error_t rc;

    printf("\n");
    printf("Evaluating filter:\n");
    print_filter_node(f);
    rc = eval_filter_node(f, result);
    printf("eval_filter_node returned '%d'\n", rc);
    if (rc == ERR_NONE) {
	printf("result:\n");
	print_filter_node(result);
    }
    printf("\n");

    return rc;
}

void
test_negation(void)
{
    filter_node_t* result = make_bool_val(TF_FALSE);
    
    printf("\n\n\n-------------------\n");
    printf("Testing bit negation\n");
    {
	filter_node_t* val = make_ulong_val(1);
	filter_node_t* op = make_bit_not(val);

	eval_and_print_filter_and_result(op, result);

	free_filter_node(op);
    }
    printf("Testing logical negation\n");
    {
	filter_node_t* val = make_long_val(0);
	filter_node_t* op = make_logical_not(val);

	eval_and_print_filter_and_result(op, result);
	
	free_filter_node(op);
    }
    free_filter_node(result);
}
    
/* kernel variables assignment testing. */
void
test_variables(void)
{
    filter_node_t* result = make_bool_val(TF_FALSE);
    exp_error_t rc;

    printf("\n\n\n-------------------\n");
    printf("Testing variables\n");
    {
	filter_node_t* v_sys_id = make_int_val(10);
	filter_node_t* v_param_0 = make_string_val("/etc/passwd");
	filter_node_t* f0 = make_var_val(VT_SYSCALL_ID, DT_INT);
	filter_node_t* f1 =
	    make_var_indexed_val(VT_PARAM, 0, DT_STRING);

	reset_all_vars();
	set_var(VT_SYSCALL_ID, v_sys_id);
	set_var_indexed(VT_PARAM, 0, v_param_0);

	rc = eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	print_filter_node(result);

	rc = eval_filter_node(f1, result);
	printf("param 0 (%d):\n", rc);
	print_filter_node(result);
	
	free_filter_node(f1);
	free_filter_node(f0);
    }
    {
	exp_error_t rc;
	filter_node_t* v_param_0 = make_filter_vector(2);
	filter_node_t* f0 = make_string_val("/bli/bla");
	filter_node_t* f1 = make_int_val(7);
	filter_node_t* filter_0_0 =
	    make_var_indexed_field_val(VT_PARAM, 0, 0, DT_STRING);
	filter_node_t* filter_0_1 =
	    make_var_indexed_field_val(VT_PARAM, 0, 1, DT_STRING);
	insert_filter_node(v_param_0, 0, f0);
	insert_filter_node(v_param_0, 1, f1);

	reset_all_vars();
	set_var_indexed(VT_PARAM, 0, v_param_0);

	rc = eval_filter_node(filter_0_0, result);
	printf("param 0.0 (%d):\n", rc);
	print_filter_node(result);

	rc = eval_filter_node(filter_0_1, result);
	printf("param 0.1 (%d):\n", rc);
	print_filter_node(result);

	free_filter_node(filter_0_0);
	free_filter_node(filter_0_1);
    }

    free_filter_node(result);
}

/* check short circuiting */
void
test_short_circuiting(void)
{
    filter_node_t* result = make_bool_val(TF_FALSE);
    exp_error_t rc;

    printf("\n\n\n-------------------\n");
    printf("Testing short-circuiting\n");

    {
	filter_node_t* f0 = make_int_val(0);
	filter_node_t* f1 = make_int_val(1);
	filter_node_t* f_and = make_logical_and(f0, f1);

	printf("short circuiting support test: and\n");
	rc = eval_and_print_filter_and_result(f_and, result);
 	
	free_filter_node(f_and);
    }
    {
	filter_node_t* f0 = make_int_val(1);
	filter_node_t* f1 = make_int_val(0);
	filter_node_t* f_and = make_logical_and(f0, f1);

	printf("short circuiting support test: and\n");
	rc = eval_and_print_filter_and_result(f_and, result);
 	
	free_filter_node(f_and);
    }
    {
	filter_node_t* f0 = make_int_val(1);
	filter_node_t* f1 = make_int_val(0);
	filter_node_t* f_or = make_logical_or(f0, f1);

	printf("short circuiting support test: or\n");
	rc = eval_and_print_filter_and_result(f_or, result);
 	
	free_filter_node(f_or);
    }
    {
	filter_node_t* f0 = make_int_val(0);
	filter_node_t* f1 = make_int_val(1);
	filter_node_t* f_or = make_logical_or(f0, f1);

	printf("short circuiting support test: or\n");
	rc = eval_and_print_filter_and_result(f_or, result);
 	
	free_filter_node(f_or);
    }

    free_filter_node(result);
}	

/* filter evaluation testing */
void
test_filter_eval_bit(void)
{
    filter_node_t* result = make_bool_val(TF_FALSE);
    exp_error_t rc;

    {
	filter_node_t* f0 = make_long_val(120L);
	filter_node_t* f1 = make_long_val(30L);
	filter_node_t* f2 = make_bit_and(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    {
	filter_node_t* f0 = make_long_val(120L);
	filter_node_t* f1 = make_long_val(30L);
	filter_node_t* f2 = make_bit_or(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    {
	filter_node_t* f0 = make_long_val(120L);
	filter_node_t* f1 = make_long_val(30L);
	filter_node_t* f2 = make_bit_xor(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }

    free_filter_node(result);
}

/* test unsigned data types. */
void
test_unsigned_types(void)
{
    filter_node_t* result = make_bool_val(TF_FALSE);
    exp_error_t rc;

    printf("\n\n\n");
    printf("Testing unsigned data types\n\n");

    {
	filter_node_t* f0 = make_ulong_val(120L);
	filter_node_t* f1 = make_ulong_val(30L);
	filter_node_t* f2 = make_math_plus(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    {
	filter_node_t* f0 = make_ushort_val(120);
	filter_node_t* f1 = make_ushort_val(30);
	filter_node_t* f2 = make_math_plus(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    {
	filter_node_t* f0 = make_uint_val(120);
	filter_node_t* f1 = make_uint_val(30);
	filter_node_t* f2 = make_math_plus(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    {
	filter_node_t* f0 = make_ushort_val(65000);
	filter_node_t* f1 = make_uint_val(100820);
	filter_node_t* f2 = make_math_plus(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    {
	filter_node_t* f0 = make_short_val(65000);
	filter_node_t* f1 = make_uint_val(100820);
	filter_node_t* f2 = make_math_plus(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    {
	filter_node_t* f0 = make_ushort_val(64003);
	filter_node_t* f1 = make_short_val(5);
	filter_node_t* f2 = make_bit_left_shift(f0, f1);

	rc = eval_and_print_filter_and_result(f2, result);

	free_filter_node(f2);
    }
    printf("\n\n\n");
    free_filter_node(result);
}

/* test filter node vectors. */
void
test_filter_node_vectors(void)
{
    printf("\n\n\n");
    printf("Testing filter node vectors\n\n");

    {
	int rc;
	filter_node_t* f0 = make_ulong_val(120L);
	filter_node_t* f1 = make_ulong_val(30L);
	filter_node_t* f2 = make_math_plus(f0, f1);
	filter_node_t* f3 = make_bool_val(0);
	filter_node_t* f4 = make_int_val(10);
	filter_node_t* f5 = make_int_val(20);
	filter_node_t* f6 = make_equal(f4, f5);
	filter_node_t* vec = make_filter_vector(5);

	rc = insert_filter_node(vec, 0, f2);
	rc = insert_filter_node(vec, 1, f3);
	rc = insert_filter_node(vec, 2, f6);

	print_filter_node(vec);

	free_filter_node(vec);
    }
    printf("\n\n\n");
}

/* test in match. */
void
test_in_match(void)
{
    printf("\n\n\n");
    printf("Testing in match\n\n");

    {
	int rc;
	filter_node_t* result = make_bool_val(TF_FALSE);
	filter_node_t* f1 = make_int_val(10);
	filter_node_t* f2 = make_int_val(20);
	filter_node_t* f3 = make_int_val(20);
	filter_node_t* f4 = make_int_val(30);
	filter_node_t* f5 = make_int_val(40);
	filter_node_t* f6 = make_equal(f3, f4);
	filter_node_t* vec2 = make_filter_vector(2);
	filter_node_t* f7 = make_equal(f5, vec2);
	filter_node_t* vec = make_filter_vector(5);
	filter_node_t* f10 = make_int_val(20);
	filter_node_t* in = make_in_match(f10, vec);

	rc = insert_filter_node(vec, 0, f1);
	rc = insert_filter_node(vec, 1, f2);
	rc = insert_filter_node(vec, 2, f6);
	rc = insert_filter_node(vec, 3, f7);

	print_filter_node(vec);

	rc = eval_filter_node(in, result);
	print_filter_node(result);
	free_filter_node(in);
	free_filter_node(result);
    }
    printf("\n\n\n");
}

filter_node_t* callback_syscall_id (var_type_t type, void *data) {
	filter_node_t* f0 = make_string_val((char*)data);
	printf("syscall_id callback function is called.\n");
	return f0;
}

filter_node_t* callback_param (var_type_t type, int param_index, data_type_t dtype, void *data) {
	filter_node_t* f0 = make_string_val((char*)data);
	printf("callback_param() is me.\n");
	return f0;
}

void
test_logging(void)
{
    action_t* a1 = NULL;
    filter_node_t* proc_uid = make_long_val((long)0);
    filter_node_t* proc_euid = make_long_val((long)1);
    filter_node_t* proc_suid = make_long_val((long)2);
    filter_node_t* proc_gid = make_long_val((long)100);
    filter_node_t* proc_egid = make_long_val((long)101);
    filter_node_t* proc_sgid = make_long_val((long)102);
    filter_node_t* proc_pid = make_long_val((long)980);
    filter_node_t* proc_comm = make_string_val("blabla");
    char *param0 = "param 0 is a cow";

    printf("\n\n\n");
    printf("Testing logging\n\n");

    reset_all_vars();
    set_var_indexed_callback(VT_PARAM, 0, callback_param, DT_STRING, param0);
    set_var(VT_PROC_UID, proc_uid);
    set_var(VT_PROC_EUID, proc_euid);
    set_var(VT_PROC_SUID, proc_suid);
    set_var(VT_PROC_GID, proc_gid);
    set_var(VT_PROC_EGID, proc_egid);
    set_var(VT_PROC_SGID, proc_sgid);
    set_var(VT_PROC_PID, proc_pid);
    set_var(VT_PROC_COMM, proc_comm);

    a1 = make_action_log(4, "invoked: %pid[%comm]: %sid_%sname(%params)\n");
    
    tracker_perform_action(NULL, a1, 1, 10, "unlink", "P", 0);
    tracker_perform_action(NULL, a1, 2, 5, "open", "s P i i", "passwd", 0, 32768, 15);
    tracker_perform_action(NULL, a1, 3, 10, "unlink", "s S uh", "passwd", a1, 64000);

    printf("\n\n\n");
    
    free_action(a1);
    reset_all_vars();
}

void
test_var_callback(void)
{
    filter_node_t* result = make_bool_val(TF_FALSE);
    exp_error_t rc;

    printf("\n\n\n-------------------\n");
    printf("Testing variables\n");
    {
	filter_node_t* f0 = make_var_val(VT_SYSCALL_ID, DT_INT);
	filter_node_t* f1 =
	    make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	char *string = "callback var test";
	char *param0 = "param 0 is a cow";
	filter_node_t* f2 = make_string_val(string);
	filter_node_t* f3 = make_equal(f0, f2);

	reset_all_vars();
	set_persistent_callback(VT_SYSCALL_ID, callback_syscall_id, string);
	set_var_indexed_callback(VT_PARAM, 0, callback_param, DT_STRING, param0);
	
	rc = eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	print_filter_node(result);

	rc = eval_filter_node(f1, result);
	printf("param[0] (%d):\n", rc);
	print_filter_node(result);

	rc = eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	print_filter_node(result);

	rc = eval_filter_node(f1, result);
	printf("param[0] (%d):\n", rc);
	print_filter_node(result);

	rc = eval_filter_node(f3, result);
	printf("equal (%d):\n", rc);
	print_filter_node(result);

	reset_all_vars();
	
	rc = eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	print_filter_node(result);

	reset_all_vars();
	
	free_filter_node(f1);
	free_filter_node(f3);
    }
    free_filter_node(result);
}

int
main(int argc, char* argv[])
{
    struct timeval t1, t2;
    filter_node_t* result = make_bool_val(TF_FALSE);
    exp_error_t rc;
    filter_node_t* f0 = make_string_val("pw");
    filter_node_t* vec = make_filter_vector(2);
    filter_node_t* v0 = make_string_val("passwd");
    filter_node_t* v1 = make_string_val("/etc/nsswitch");
    filter_node_t* f2;
    filter_node_t* f3;
    filter_node_t* f4;
    rule_list_t* rules;

    insert_filter_node(vec, 0, v0);
    insert_filter_node(vec, 1, v1);
    f2 = make_in_match(f0, vec);
    f3 = make_int_val(15);
    f4 = make_logical_and(f2, f3);

    printf("Printing filter node.\n");
    print_filter_node(f4);


    gettimeofday(&t1, NULL);
    rc = eval_filter_node(f4, result);
    gettimeofday(&t2, NULL);

    printf("   time before eval: %ld, %ld\n", t1.tv_sec, t1.tv_usec);
    printf("   time after  eval: %ld, %ld\n", t2.tv_sec, t2.tv_usec);
    printf("Expression result (rc = %d):\n", rc);
    print_filter_node(result);

    /* filter serialization/deserializatin testing. */
    {
	unsigned char buf[1000];
	int len = serialize_filter_node(f4, buf, 1000);
	printf("serialized 'f4' into '%d' bytes\n", len);

	if (len > 0) {
	    filter_node_t* f_deser =
		deserialize_filter_node(buf, len);
	    if (f_deser) {
		printf("'f4' deserialized:\n");
		print_filter_node(f_deser);
		free_filter_node(f_deser);
	    }
	    else {
		printf("Deserialization of 'f4' failed.\n");
	    }
	}
    }
	
    /* tracking rules serialization/de-serialization testing. */
    {
	unsigned char buf[1000];
	action_t* a1 = make_action_log(4, NULL);
	int len = serialize_tracking_rule(1, RW_BEFORE, 10, a1, f4, buf, 1000);
	printf("serialized rule 1 into '%d' bytes\n", len);

	if (len > 0) {
	    action_t* a_deser;
	    filter_node_t* f_deser;
	    int rule_id;
	    rule_when_t when;
	    int syscall_id;
	    int rc = deserialize_tracking_rule(buf,
					       len,
					       &rule_id,
					       &when,
					       &syscall_id,
					       &a_deser,
					       &f_deser);
	    if (rc > 0) {
		printf("rule 1 deserialized:\n");
		printf(" rule_id - %d\n", rule_id);
		printf(" when - %d (%s)\n", when, when ? "after" : "before");
		printf(" syscall_id - %d\n", syscall_id);
		print_action_node(a_deser);
		print_filter_node(f_deser);
		free_action(a_deser);
		free_filter_node(f_deser);
	    }
	    else {
		printf("Deserialization of rule 1 failed.\n");
	    }
	}
	
	free_action(a1);
    }

    /* tracking rules add/delete testing. */
    {
	action_t* a1 = make_action_log(4, NULL);
	action_t* a2 = make_action_disable_rule(6);
	int rc;
		
	printf("creating tracking rules list\n");
	rules = create_tracking_rule_list();
	printf("adding tracking rules\n");
	rc = add_tracking_rule(rules, 6, f4, a1);
	/*rc = add_tracking_rule(rules, 3, result, a2);*/

	print_tracking_rules(rules);

		
	printf("deleting all tracking rules\n");
	del_all_tracking_rules_in_list(rules);
	print_tracking_rules(rules);
	delete_tracking_rule_list(rules);

	free_action(a1);
	free_action(a2);
    }

    test_variables();

    test_short_circuiting();

    test_filter_eval_bit();

    test_unsigned_types();
	
    test_filter_node_vectors();

    test_negation();

    test_in_match();

    test_var_callback();

    test_logging();

    free_filter_node(result);

    return 0;
}
