/*
 * $Id: request_buffer.c,v 1.1 2001/12/24 21:04:57 mulix Exp $
 */
#include <linux/errno.h>
#include <asm/uaccess.h>

#include "request_buffer.h"
#include "module_interface.h"

/* add to the end of the request buffer this user supplied chunk,  */
/* with this size.  */
/* returns 0 on succes, a negative error on failure.  */
/* NOTE: chunk should point into user space memory, add_chunk will  */
/* take care of the copying */
int rbuf_add_chunk(req_buf_t buf, unsigned char* chunk, size_t sz)
{
    if (!buf)
	return -EINVAL;

    /* check that we have enough free space */
    if (sz > buf->free_space)
	return -ENOSPC; /* TODO: in the future, enlarge the buffer */
    
    if (copy_from_user (buf->whead, chunk, sz)) {
        return -EFAULT;
    }

    buf->whead += sz;
    buf->data_size += sz;
    buf->free_space -= sz;
    
    return 0;
}

/* does this request buffer contain a whole request packet, or not yet? */
int rbuf_has_whole_packet(req_buf_t buf)
{
    unsigned int req_sz;
    
    if (!buf)
	return -EINVAL;

    /* get the size of this request packet, which is the first 'int' in */
    /* the data buffer */
    req_sz = *(int*)buf->data;
   
    /* now, sanity checks on req_sz */
    /* if it's bigger than the entire space we have, this packet is invalid */
    if (req_sz > RBUF_ORIG_BUF_SIZE)
	return -EINVAL;

    /* we dont have quite enough data yet */
    if (req_sz > buf->data_size)
	return 0; /* not a full packet yet */
    
    /* ok, so req_sz must be smaller or equal to the data_size. cool! */
    
    /* let's check the last int from the end, if it's the magic cookie */
    /* we expect */
    if (buf->data[req_sz-1] != DEV_FILE_MAGIC_COOKIE)
	return -EINVAL; 

    /* whee, success */
    return 1;
}

/* get the request buffer's raw data */
unsigned char* rbuf_get_data(req_buf_t buf)
{
    return buf->data;
}

/* clear this request buffer */
void rbuf_clear(req_buf_t buf)
{
    if (!buf)
	return;

    /* set the write head to point to the beginning of the data buf */
    buf->whead = buf->data;
    /* we have this much free space. note: buf->data is a char array. if 
       we change it to char ptr, need to keep its size explicitly */
    buf->free_space = RBUF_ORIG_BUF_SIZE;
    buf->data_size = 0;
    /* empty the buffer, just in case */
    memset(buf->whead, 0, buf->free_space);
}
