/*
 * $Id: var_callbacks.c,v 1.1 2001/12/20 04:18:04 mulix Exp $
 */ 

#include <linux/kernel.h>
#include <linux/sched.h>
#include <asm/current.h> /* for current */

#include "rule_engine_public.h"
#include "string_util.h"


 
/*
 * returns number of ancestors.
 * if vec isn't null, it also sets the processes nodes
 * for ancestors var.
 */

static int read_ancestors_into_vector(filter_node_t *vec)
{
	struct task_struct *t;
	filter_node_t *f;
	int rc = 0;
  
	read_lock(&tasklist_lock);
	for (t = current->p_pptr; t; t = t->p_pptr, rc++)
	{
		if (vec) {
			f = make_filter_vector(2);
			if (f) {
				/* insert maybe fail, because if number of ancestors became
				 * bigger while two read_ancestors_into_vector calls
				 */
				if (insert_filter_node(vec, rc, f) == 1) {
					insert_filter_node(f, 0, make_long_val((long)t->pid));
					insert_filter_node(f, 1, make_string_val(t->comm));
				} else {
					free_filter_node(f);
					/* we dont fail the call, because the vector is still
					 * filled with valid inforamtion
					 */
					break;
				}
			} else {
				rc = 0;
				break;
			}
		}
	}
	read_unlock(&tasklist_lock);
	return rc;
}
                                                                        
/* variable calculation callbacks. */
filter_node_t* proc_current_callback(var_type_t type, void *data) {
	filter_node_t *f = NULL;
    
	switch (type) {
	case VT_PROC_UID:
		f = make_long_val((long)current->uid);
		break;
	case VT_PROC_EUID:
		f = make_long_val((long)current->euid);
		break;
	case VT_PROC_SUID:
		f = make_long_val((long)current->suid);
		break;
	case VT_PROC_GID:
		f = make_long_val((long)current->gid);
		break;
	case VT_PROC_EGID:
		f = make_long_val((long)current->egid);
		break;
	case VT_PROC_SGID:
		f = make_long_val((long)current->sgid);
		break;
	case VT_PROC_PID:
		f = make_long_val((long)current->pid);
		break;
	case VT_PROC_COMM:
		f = make_string_val(current->comm);
		break;
	case VT_PROC_PPID:
	case VT_PROC_PCOMM:
	{
		filter_node_t *ppid = NULL, *pcomm = NULL;
                                                                        
		read_lock(&tasklist_lock);
		if (current->p_pptr) {
			ppid = make_long_val((long)current->p_pptr->pid);
			pcomm = make_string_val(current->p_pptr->comm);
		}
		read_unlock(&tasklist_lock);
		if (type == VT_PROC_PPID) {
			f = ppid;
			set_var(VT_PROC_PCOMM, pcomm);
		} else {
			f = pcomm;
			set_var(VT_PROC_PPID, ppid);
		}
		break;
	}
	case VT_PROC_ANCESTORS:
	{
		int p;
                                                                         
		p = read_ancestors_into_vector(NULL);
		if (p && (f = make_filter_vector(p))) {
			p = read_ancestors_into_vector(f);
			if (!p) {
				free_filter_node(f);
				f = NULL;
			}
		}
	}
	default:
		f = NULL;
		break;
	}
	return f;
}

filter_node_t* global_retval_callback(var_type_t type, void *data)
{
	filter_node_t *f = NULL;
    
	switch (type) {
	case VT_RETVAL:
		f = make_int_val((int)data);
		break;
	default:
		f = NULL;
		break;
	}

	return f;
}

filter_node_t* global_param_callback(var_type_t type, int param_index, data_type_t dtype, void *data) 
{
	filter_node_t *f = NULL;
    
	switch (dtype) {
	case DT_CHAR:
		f = make_char_val(*((char*)data));
		break;
	case DT_BOOL:
	case DT_INT:
		f = make_int_val(*((int*)data));
		break;
	case DT_UINT:
		f = make_uint_val(*((unsigned int*)data));
		break;
	case DT_SHORT:
		f = make_short_val(*((short*)data));
		break;
	case DT_USHORT:
		f = make_ushort_val(*((unsigned short*)data));
		break;
	case DT_LONG:
		f = make_long_val(*((long*)data));
		break;
	case DT_ULONG:
		f = make_ulong_val(*((unsigned long*)data));
		break;
	case DT_FLOAT:
		f = make_ulong_val(*((float*)data));
		break;
	case DT_STRING: 
	{
		int retval = 0;
		char *string = NULL;
		retval = getstring((char*)data, &string);
		if (retval == 0)
			f = make_string_val_without_copy(string);
		break;
	}
	default:
		f = NULL;
		break;
	}
    
	return f;
}

