/* 
 * $Id: sct_rules_module_tests.c,v 1.17 2001/12/29 15:26:16 mulix Exp $
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/resource.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>


#include "tester.h"

#include "sct_ctrl_lib/sct_ctrl_lib.h"

static unsigned int rule_id = 0;
extern sct_ctrl_t ctrl;
extern char* progname;

int open_test(void)
{
    int fd = open("tmp",O_CREAT,S_IWUSR);
    if (fd != -1)
	close(fd);
    return 1;
}

int unlink_test(void)
{
    (void)unlink("tmp");
    (void)link(progname, "tmplink2");
    (void)unlink("tmplink2");
    return 1;
}

int link_test(void)
{
    (void)link(progname, "tmplink");
    return 1;
}
    
int chdir_test(void) 
{
    static char cwd[1024];
    getcwd(cwd, sizeof(cwd));

    if (chdir("tmpdir") == 0)
        (void)chdir("..");
    
    return 1; 
}

int chmod_test(void)
{
    (void) chmod("tmplink", S_IXUSR);
    return 1;
}

int access_test(void)
{
    (void)access(progname, X_OK);
    return 1;
}

int mkdir_test(void)
{
    (void)mkdir("tmpdir", S_IRWXU);
    (void)rmdir("tmpdir");
    (void)mkdir("tmpdir", S_IRWXU);
    return 1;
}

int rmdir_test(void)
{
    const char location[] = "tmpdir";
    (void)rmdir(location);
    return 1;
}

int getpid_test(void)
{
    (void)getpid();
    return 1;
}

int chroot_test(void)
{
    (void)chroot("/");
    return 1; 
}

int rename_test(void)
{
    (void)creat("tmprename", S_IRUSR | S_IWUSR);
    (void)rename("tmprename","tmp");
    return 1;
}

int nice_test(void)
{
    (void)nice(1);
    return 1;
}           

int dup_test(void)            
{
    int fd = dup(1);
    if (fd != -1)
	close(fd);

    return 1;
}

int dup2_test(void)
{
    int fd = fileno(stdout);
    if ((fd = dup2(fd, fileno(stderr))) != -1) {
	close(fd);
    }
    return 1; 
}



int symlink_test(void)
{
    (void)symlink(progname, "tmpsymlink");
    return 1;
}

int getpriority_test(void)
{
    (void)getpriority(PRIO_PROCESS, getpid());
    return 1;
}

int setpriority_test(void)
{
    (void)setpriority(PRIO_PROCESS,getpid(),1);
    return 1;
}

int lseek_test(void)
{
    int fd;
    if ((fd = open("tmp", O_CREAT, S_IWUSR)) != -1){
	(void)lseek(fd,3,SEEK_CUR);
	close(fd);
    }
    return 1;
}

int socket_test(void)
{
    int fd;
    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) != -1){
	close(fd);
    }
    return 1;
}

int bind_test(void)
{
    int fd;
    struct sockaddr_in sa;
    struct hostent* hen;

    hen = gethostbyname("localhost");
    if (!hen)
	return 0;

    /* initiate machine's Internet address structure */
    /* first clear out the struct, to avoid garbage  */
    memset(&sa, 0, sizeof(sa));
    /* Using Internet address family */
    sa.sin_family = AF_INET;
    /* copy port number in network byte order */
    sa.sin_port = htons(35775);
    /* copy IP address into address struct */
    memcpy(&sa.sin_addr.s_addr, hen->h_addr_list[0], hen->h_length);

    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) != -1){
	bind(fd, (struct sockaddr *)&sa, sizeof(sa));
	close(fd);
    }
    return 1;
}

int listen_test(void)
{
    int fd;
    struct sockaddr_in sa;
    struct hostent* hen;

    hen = gethostbyname("localhost");
    if (!hen)
	return 0;

    /* initiate machine's Internet address structure */
    /* first clear out the struct, to avoid garbage  */
    memset(&sa, 0, sizeof(sa));
    /* Using Internet address family */
    sa.sin_family = AF_INET;
    /* copy port number in network byte order */
    sa.sin_port = htons(35776);
    /* copy IP address into address struct */
    memcpy(&sa.sin_addr.s_addr, hen->h_addr_list[0], hen->h_length);

    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) != -1){
	if (bind(fd, (struct sockaddr *)&sa, sizeof(sa)) == 0){
	    listen(fd, 5);
	}
	close(fd);
    }
    return 1;
}

#define SOCKET_CONNECT_ACCEPT_PORT 35777

int accept_test(void)
{
    int fd;
    struct sockaddr_in sa;
    fd_set accept_set;

    /* initiate machine's Internet address structure */
    /* first clear out the struct, to avoid garbage  */
    memset(&sa, 0, sizeof(sa));
    /* Using Internet address family */
    sa.sin_family = AF_INET;
    /* copy port number in network byte order */
    sa.sin_port = htons(SOCKET_CONNECT_ACCEPT_PORT);
    /* copy IP address into address struct */
    sa.sin_addr.s_addr = INADDR_ANY;

    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) != -1){
	setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, NULL, 0);
	if (bind(fd, (struct sockaddr *)&sa, sizeof(sa)) == 0){
	    if (listen(fd, 5) == 0){
		struct sockaddr_in client_addr;
		socklen_t client_addr_len;
		int cs;
		int rc;
		struct timeval timeout;

		memset(&client_addr, 0, sizeof(client_addr));
		client_addr_len = sizeof(client_addr);

    		/* check if we've got a new pending connection */
		FD_ZERO(&accept_set);
		FD_SET(fd, &accept_set);
		timeout.tv_sec = 10;
		timeout.tv_usec = 0;
		rc = select(fd+1, &accept_set, NULL, NULL, &timeout);
		if (rc == 1 && FD_ISSET(fd, &accept_set)) {
		    cs = accept(fd, (struct sockaddr *)&client_addr, &client_addr_len);
		    if (cs >= 0)
		        close(cs);
		}
	    }
	}
	close(fd);
    }
    return 1;
}

int connect_test(void)
{
    int fd;
    struct sockaddr_in sa;

    /* initiate machine's Internet address structure */
    /* first clear out the struct, to avoid garbage  */
    memset(&sa, 0, sizeof(sa));
    /* Using Internet address family */
    sa.sin_family = AF_INET;
    /* copy port number in network byte order */
    sa.sin_port = htons(SOCKET_CONNECT_ACCEPT_PORT);
    /* copy IP address into address struct */
    inet_aton("127.0.0.1", &sa.sin_addr);

    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) != -1){
	int rc;

	rc = connect(fd, (struct sockaddr *)&sa, sizeof(sa));
	if (rc == -1) {
	   /*perror("connect_test: connect:");*/
	}
	shutdown(fd, 1);
	close(fd);
    }
    else {
	/*perror("connect_test: socket:");*/
    }
    return 1;
}

static void
cleanup(void)
{
    (void)unlink("tmplink");
    (void)unlink("tmpsymlink");
    (void)unlink("tmp");
}    

static void
register_module_passwd_rule(unsigned int rid)
{
    filter_node_t* f0 = make_var_indexed_val(VT_PARAM, 0, DT_STRING);
    filter_node_t* f1 = make_string_val("passwd");
    filter_node_t* f2 = make_pattern_match(f0, f1);
    action_t* a1 = make_action_log(4, NULL);
    
    if (!sct_ctrl_add_rule(ctrl, rid, RW_BEFORE, 10, f2, a1, 1))
	die ("failed to upload rule '%d'\n", rid);
}

int
test_simple_syscall_usage(void)
{
    unsigned int num_tests = 100;
    int rc = fork();
    if (rc == -1) {
	perror("test_simple_syscall_usage: fork failed:");
	die("'%s': fork failed\n", __FUNCTION__);
    }

    /* both father and child execute this */ 
    while (--num_tests){ 
	int delay = 1+(int) (10.0 * rand() /(RAND_MAX+1.0));
	creat("passwd", S_IRUSR | S_IWUSR);
	usleep(delay);
	remove("passwd");
    }
    
    if (rc == 0) /* child */
	_exit(0);

    return 1;
}

int
test_settimeofday_usage(void)
{
    unsigned int num_tests = 100;
    int rc;
    int rid = ++rule_id;

    filter_node_t* timeval_match = make_bool_val(1);
    if (!sct_ctrl_add_rule(ctrl, rid, RW_BEFORE, 79, timeval_match, make_action_log(4, NULL), 1))
	die("failed to upload rule '%d'\n", rid);

    rc = fork();
    if (rc == -1) {
	perror("test_settimeofday_usage: fork failed:");
	die("'%s': fork failed\n", __FUNCTION__);
    }

    /* both father and child execute this */ 
    while (--num_tests){ 
	struct timeval tv = {10, 1000 - num_tests};
	int delay = 1+(int) (10.0 * rand() /(RAND_MAX+1.0));
	settimeofday(&tv, NULL);
	usleep(delay);
    }
    
    if (rc == 0) /* child */
	_exit(0);

    if (!sct_ctrl_del_rule(ctrl, rid, RW_BEFORE, 79))
	die("failed to delete rule '%d'\n", rid);
    
    return 1;
}


int 
test_rule_register_deregister(void)
{
    unsigned int num_tests = 100;
    int rc;

    if (!sct_ctrl_del_all_rules(ctrl))
	die("'%s': failed to delete all module rules\n");

    rc = fork();
    if (rc == -1) {
	perror("test_rule_register_deregister: fork failed:");
	die("'%s': fork failed\n", __FUNCTION__);
    }

    if (rc == 0){ /* child code */
	++rule_id;
    }
    
    /* both father and child execute this */ 
    while (--num_tests){ 
	int delay = 1+(int) (5.0 * rand() /(RAND_MAX+1.0));
	unsigned int rid = (rule_id += 2);
	register_module_passwd_rule(rid);
	usleep(delay);
	if (!sct_ctrl_del_rule(ctrl, rid, RW_BEFORE, 10))
	    die ("'%s': failed to delete rule\n", __FUNCTION__);
	if (rc == 0) {
	    printf("."); fflush(stdout);
	}
    }
    
    if (rc == 0) /* child */
	_exit(0);

    printf("\n"); fflush(stdout); 

    sct_ctrl_print_all_rules(ctrl);

    return 1;    
}

static void 
upload_rules(sct_ctrl_t ctrl, int syscall_id, filter_node_t* f)
{
    action_t* a1 = make_action_log(4, NULL);
    action_t* a2 = make_action_log(4, "after syscall: %pid[%comm]: %sid_%sname(%params) = %retval (rule %ruleid)");

    if (a1 && a2) {
	if (!sct_ctrl_add_rule(ctrl, ++rule_id, RW_BEFORE, syscall_id, f, a1, 1))
	    die ("failed to upload rule '%d'\n", rule_id);
	
	if (!sct_ctrl_add_rule(ctrl, ++rule_id, RW_AFTER, syscall_id, f, a2, 1))
	    die ("failed to upload rule '%d'\n", rule_id);
	
    }
    free_action(a1);    
    free_action(a2);
}

static void 
all_syscalls_register_rules(void)
{
    filter_node_t* p0 = make_var_indexed_val(VT_PARAM, 0, DT_STRING);
    filter_node_t* p1 = make_var_indexed_val(VT_PARAM, 1, DT_STRING);

    filter_node_t* prog = make_string_val(progname);
    filter_node_t* progmatch0 = make_pattern_match(p0, prog);
    /* filter_node_t* progmatch1 = make_pattern_match(p1, prog); */

    filter_node_t* tmp = make_string_val("tmp");
    filter_node_t* tmpmatch0 = make_pattern_match(p0, tmp);
    filter_node_t* tmpmatch1 = make_pattern_match(p1, tmp);

    filter_node_t* test = make_string_val("tests");
    filter_node_t* testmatch0 = make_pattern_match(p0, test);

    filter_node_t* truematch = make_bool_val(1);

    filter_node_t* intmatch0 = make_var_indexed_val(VT_PARAM, 0, DT_INT);
    filter_node_t* intmatch1 = make_int_val(1);
    filter_node_t* intmatch2 = make_equal(intmatch0, intmatch1);

    filter_node_t* matchnum0 = make_var_indexed_val(VT_PARAM, 1, DT_INT);
    filter_node_t* matchnum1 = make_int_val(3);
    filter_node_t* matchnum2 = make_equal(matchnum0, matchnum1);
    filter_node_t* true_match =make_bool_val(1);

    filter_node_t* timeval_match = make_bool_val(1);

    upload_rules(ctrl, 5, tmpmatch0);
    upload_rules(ctrl, 9, tmpmatch1);
    upload_rules(ctrl, 10, tmpmatch0);
    upload_rules(ctrl, 12, testmatch0);
    upload_rules(ctrl, 15, tmpmatch0);
    upload_rules(ctrl, 19, matchnum2);
    upload_rules(ctrl, 20, truematch);
    upload_rules(ctrl, 33, progmatch0);
    upload_rules(ctrl, 34, intmatch2);

    /* TODO: add 'kill' rule */

    upload_rules(ctrl, 38, tmpmatch1);
    upload_rules(ctrl, 39, tmpmatch0);
    upload_rules(ctrl, 40, tmpmatch0);
    upload_rules(ctrl, 41, intmatch2);
    upload_rules(ctrl, 61, tmpmatch0);
    upload_rules(ctrl, 63, intmatch2);
    upload_rules(ctrl, 79, timeval_match);
    upload_rules(ctrl, 83, tmpmatch1);
    upload_rules(ctrl, 96, intmatch2);
    upload_rules(ctrl, 97, intmatch2);

    /* test 'socket' call. */
    upload_rules(ctrl, 65638, true_match);

    /* test 'bind' call. */
    upload_rules(ctrl, 131174, true_match);

    /* test 'listen' call. */
    upload_rules(ctrl, 262246, true_match);

    /* test 'accept' call. */
    upload_rules(ctrl, 327782, true_match);

    /* test 'connect' call. */
    upload_rules(ctrl, 196710, true_match);

    /* test 'shutdown' call. */
    upload_rules(ctrl, 852070, true_match);

    sct_ctrl_print_all_rules(ctrl);
    sleep(3);
}

static void 
all_syscalls_deregister_rules(void)
{
    if (!sct_ctrl_del_all_rules(ctrl))
	die("failed to delete all module rules");
}

int
test_all_syscalls_usage(void)
{
    typedef struct syscall_struct_t {
    unsigned int syscall_num;
    int (*ptest)(void);
    } syscall_struct;

    syscall_struct syscall_test[] =
    {
        {5, open_test}, {9, link_test}, {10, unlink_test},
        {15, chmod_test}, {19, lseek_test}, {20, getpid},
	{33, access_test}, {34, nice_test},
        {38, rename_test}, {39, mkdir_test}, {12, chdir_test},
        {61, chroot_test}, {40, rmdir_test}, {41, dup_test},
        {63, dup2_test}, {83, symlink_test}, 
        {96, getpriority_test}, {97, setpriority_test},
	{65638, socket_test}, {131174, bind_test}, {262246, listen_test},
	{327782, accept_test}, {196710, connect_test}
    };

    const unsigned int num_syscalls = sizeof(syscall_test)/sizeof(syscall_struct) ;
    unsigned int times_to_run = 30;
    unsigned int i;
    int rc;

    all_syscalls_register_rules();

    for (i = 0; i < num_syscalls; ++i){
        rc = fork();
        if (rc == -1) {
	    perror("test_all_syscalls_usage: fork failed:");
            die("'%s': fork failed\n", __FUNCTION__);
	}
        if (rc == 0)
            break;
    }
    /* parent code */
    if (rc != 0){
	/* we have as many children as there are syscalls
	   a relic from the days when each child perform 
	   only one test */
        unsigned int children = num_syscalls;
        while (children--)
            (void)wait(NULL);
	
    } else { /* child code */
        
	while (--times_to_run){
            int delay = 1+(int) (10.0 * rand() /(RAND_MAX+1.0));
            usleep(delay);

            for (i = 0; i < num_syscalls; ++i) 
            {
	        rc = syscall_test[i].ptest();
            }
	    if (times_to_run % 10 == 0) {
	        printf("."); fflush(stdout);
	    }
        }	
   	fprintf(stderr, "\nchild '%d' finished\n", i);
        _exit(0); 
    }
    
    cleanup();  

    printf("\n"); fflush(stdout); 

    all_syscalls_deregister_rules();

    return 1;
}

