/* 
 * $Id: tester.c,v 1.16 2001/12/20 04:18:04 mulix Exp $
 */ 

/* 
 * a testing program for the kernel module. to add more tests, write a
 * test function and add a call to it in main. your funtion should return
 *   0 if the test was succesful and anything else otherwise. 
 */

#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>

#include <assert.h>

#include "sct_ctrl_lib/sct_ctrl_lib.h"
#include "sct_rules/rule_engine_public.h"

#include "tester.h"

const char* progname;
sct_ctrl_t ctrl;

void 
_die(const char* func, const char* file,
     unsigned int line, const char* fmt, ...)
{
    va_list ap; 

    fprintf(stdout, "[%s:%d:%s] ",file, line, func);
    
    va_start(ap, fmt);
    vfprintf(stdout, fmt, ap);
    va_end(ap);

    assert(0);
}

void 
_logerr(const char* func, const char* file,
	unsigned int line, const char* fmt, ...)
{
    va_list ap; 
    
    printf("[%s:%d:%s] ",file, line, func);

    va_start(ap, fmt);
    vfprintf(stdout, fmt, ap);
    va_end(ap);
}

static void 
setup(void)
{
    /* setup module */
    ctrl = sct_ctrl_init();
    if (!ctrl){
	die("failed to init sct_ctrl_lib\n");
    }
    
    /* del module rules */
    if (!sct_ctrl_del_all_rules(ctrl)){
	die("failed to delete all rules\n");
    }
}


int 
main(int argc, char* argv[])
{
    progname = argv[0];

    setup();
    
    fprintf(stderr, "starting sct_rules tests\n");

    /* sct_rules tests */

    TEST(test_filter_eval_bit());
    
    TEST(test_eval_complex_filter());
    
    TEST(test_filter_serialize_deserialize());
    
    TEST(test_rules_serialize_deserialize());
    
    TEST(test_tracking_rule_add_delete());
    
    TEST(test_kernel_variables());
    
    TEST(test_short_circuiting());
    
    fprintf(stderr, "starting module tests\n");

    TEST(test_simple_syscall_usage());

    TEST(test_rule_register_deregister());
     
    TEST(test_all_syscalls_usage());

    /* disable this test by default, since it can be destructive */
/*     TEST(test_settimeofday_usage());  */

    fprintf(stderr, "tests ended succesfully!\n");

    sct_ctrl_cleanup(ctrl);

    return 0;
}
